/**
 * Created by IntelliJ IDEA.
 * User: mopar
 * Date: Sep 14, 2006
 * Time: 2:07:02 AM
 * To change this template use File | Settings | File Templates.
 */
// $Id$

import java.io.File;
import java.io.FileOutputStream;

/**
 * Class to replace the functionality of the Sun JDK.
 * Currently implements a replacement for javac, javadoc, and jar.
 *
 * @author moparisthebest
 */
public class JDKClass {

    private String homeDir;

    /**
     * Default Constructor
     */
    public JDKClass() {

    }

    /**
     * Default Constructor
     */
    public JDKClass(String homeDir) {
        this.homeDir = homeDir;
    }

    /**
     * Gets the current home directory.
     *
     * @return Home Directory
     */
    public String getHomeDir() {
        return homeDir;
    }

    /**
     * Sets the home directory.
     *
     * @param homeDir Home Directory
     */
    public void setHomeDir(String homeDir) {
        this.homeDir = homeDir;
    }


    /**
     * Searches a directory recursively for java source files.
     *
     * @param dir Absolute path of directory.
     * @return String of all the .java files in the directory recursively, deliminated by ;
     */
    private static String findJavaRecursive(String dir) {
        String javafiles = findJavaRecursive(new File(dir));
        if (javafiles.length() > 1)
            javafiles = javafiles.substring(0, javafiles.length() - 1);
        return javafiles;
    }

    /**
     * Searches a directory recursively for java source files.
     *
     * @param srcDir Directory File.
     * @return String of all the .java files in the directory recursively, deliminated by ;
     */
    private static String findJavaRecursive(File srcDir) {
        String returnString = "";
        File[] javaFiles = srcDir.listFiles();
        for (File javafile : javaFiles) {
            if (javafile.getName().endsWith(".java") || javafile.isDirectory()) {
                if (javafile.isDirectory())
                    returnString += findJavaRecursive(javafile);
                else
                    returnString += javafile.getAbsolutePath() + ";";
            }
        }
        return returnString;
    }

    /**
     * Searches a directory recursively for any files.
     *
     * @param dirName Absolute path of directory.
     * @param param   Parameter to lead the files with, complete with deliminator ;
     * @return String of all the files in the directory recursively, prefixed by param, deliminated by ;
     */
    private String findRecursive(String dirName, String param) {
        File dir = new File(dirName);
        String returnString = "";
        File[] javaFiles = dir.listFiles();
        for (File javafile : javaFiles)
            if (javafile.isDirectory())
                returnString += findRecursive(javafile, param, javafile.getName() + "/");
            else
                returnString += param + javafile.getName() + ";";
        return returnString.substring(0, returnString.length() - 1);
    }

    /**
     * Searches a directory recursively for any files, prefixes them with the dir name.
     * This method is only meant to be called from findRecursive(String dirName, String param)
     *
     * @param dir    File to search.
     * @param param  Parameter to lead the files with, complete with deliminator ;
     * @param prefix Directory name to prefix all the files with.
     * @return String of all the files in the directory recursively, prefixed by param and prefix, deliminated by ;
     * @see JDKClass#findRecursive(String, String)
     */
    private String findRecursive(File dir, String param, String prefix) {
        String returnString = "";
        File[] javaFiles = dir.listFiles();
        for (File javafile : javaFiles)
            if (javafile.isDirectory())
                returnString += findRecursive(javafile, param, prefix + javafile.getName() + "/");
            else
                returnString += param + prefix + javafile.getName() + ";";
        return returnString;
    }

    /**
     * This compiles java source into class files using the previously set home Directory.
     *
     * @param srcDir    The subdirectory of the home directory where the source files reside.
     * @param binDir    The subdirectory of the home directory where you want to put the class files.
     * @param classPath The full classpath to look for the needed library files, deliminated by :.
     * @return True if compilation was succesful, false otherwise.
     */
    public boolean compile(String srcDir, String binDir, String classPath) {
        /*
             // Start up the compiler
             Process p = Runtime.getRuntime().exec( "javac "+javaFile );
             // Wait for it to finish running
             try {
               p.waitFor();
             } catch( InterruptedException ie ) { System.out.println( ie ); }
             // Check the return code, in case of a compilation error
             int ret = p.exitValue();
        */
        if (homeDir == null) return false;
        String params = "-cp;" + classPath + ";-d;" + homeDir + binDir + "/;" + findJavaRecursive(homeDir + srcDir);
        System.out.println("javac " + params);
        int ret = com.sun.tools.javac.Main.compile(params.split(";"));
        return ret == 0;
    }

    /**
     * This produces javadocs in the javadoc subdirectory of the home directory.
     *
     * @param srcDir The subdirectory of the home directory where the source files reside.
     * @return True if javadoc'n was succesful, false otherwise.
     */
    public boolean javaDoc(String srcDir) {
        if (homeDir == null) return false;
        String params = "-d;" + homeDir + "javadoc/;" + findJavaRecursive(homeDir + srcDir);
        System.out.println("javadoc " + params);
        int ret = com.sun.tools.javadoc.Main.execute(params.split(";"));
        return ret == 0;
    }

    /**
     * This produces a jar file in the home directory.
     *
     * @param name   Name of the jar file to create.
     * @param subDir Subdirectory of home directory of whose content you want in the jar file.
     * @return True if the jar was created succesfully, false otherwise.
     */
    public boolean jar(String name, String subDir) {
        return jar(name, subDir, null, null);
    }

    /**
     * This produces a jar file in the home directory, complete with a manifest attribute of Main-Class:
     *
     * @param name      Name of the jar file to create.
     * @param subDir    Subdirectory of home directory of whose content you want in the jar file.
     * @param mainClass Main Class to put in the manifest.
     * @return True if the jar was created succesfully, false otherwise.
     */
    public boolean jar(String name, String subDir, String mainClass) {
        return jar(name, subDir, mainClass, null);
    }

    /**
     * This produces a jar file in the home directory, complete with a manifest attribute of Main-Class:
     *
     * @param name      Name of the jar file to create.
     * @param subDir1   The first subdirectory of home directory of whose content you want in the jar file.
     * @param subDir2   The second subdirectory of home directory of whose content you want in the jar file.
     * @param mainClass Main Class to put in the manifest.
     * @return True if the jar was created succesfully, false otherwise.
     */
    public boolean jar(String name, String subDir1, String mainClass, String subDir2) {
        if (homeDir == null) return false;
        String manifestStr = "";
        if (mainClass != null) {
            try {
                File manifest = new File(homeDir + "MANIFEST.MF");
                if (!manifest.exists()) {
                    manifest.deleteOnExit();
                    FileOutputStream fos = new FileOutputStream(manifest);
                    fos.write(("Main-Class: " + mainClass + "\n\n").getBytes());
                }
                manifestStr = "cvmf;" + homeDir + "MANIFEST.MF;";
            } catch (Exception e) {
                manifestStr = "cvf;";
            }
        } else
            manifestStr = "cvf;";
        if (!name.endsWith(".jar"))
            name += ".jar";
        String params = manifestStr + homeDir + name + ";" + findRecursive(homeDir + subDir1, "-C;" + homeDir + subDir1 + ";");
        if (subDir2 != null)
            params += ";" + findRecursive(homeDir + subDir2, "-C;" + homeDir + subDir2 + ";");
        System.out.println("jar " + params);
        sun.tools.jar.Main jarer = new sun.tools.jar.Main(System.out, System.err, "jar");
        return jarer.run(params.split(";"));
    }
}
