import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

/**
 * Configuration class that dynamically holds any variable you wish in the properties file.
 *
 * @author moparisthebest
 */
public class ProjectConfig {

    private Properties config = new Properties();

    /**
     * Default Constructor.
     */
    public ProjectConfig() {

    }

    /**
     * Constructor that sets the only two required parameters of the ProjectConfig.
     *
     * @param name    The name of the project.
     * @param homeDir The absolute path of the project's home directory.
     */
    public ProjectConfig(String name, String homeDir) {
        config.setProperty("name", name);
        if (!homeDir.endsWith("/"))
            homeDir += "/";
        config.setProperty("homeDir", homeDir);
        saveSettings();
    }

    /**
     * Gets the property associated with the key.
     *
     * @param key Key of the property you want.
     * @return The property if it exists, a blank string otherwise, will not return null.
     */
    public String get(String key) {
        String prop = config.getProperty(key);
        if (prop == null)
            return "";
        else
            return prop;
    }

    /**
     * Sets a key/property pair, then saves it to a properties file.
     *
     * @param key     The key to assign the property to.
     * @param content They property to put into the specified key value.
     */
    public void set(String key, String content) {
        if (key == null || content == null) return;
        if (key.equals("homeDir") && !content.endsWith("/"))
            content += "/";
        config.setProperty(key, content);
        saveSettings();
    }

    /**
     * Loads all the settings from a properties file.
     *
     * @param file File to load from.
     * @return True if succesfull, false otherwise.
     */
    public boolean loadSettings(String file) {
        try {
            config.load(new FileInputStream(file));
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * Saves the settings of this class to a properties file.
     * the file made is homeDir/name.side
     */
    public void saveSettings() {
        try {
            config.store(new FileOutputStream(get("homeDir") + get("name") + ".side"), "Project " + get("name") + " for SIDE.");
        } catch (Exception e) {
            System.out.println("[ERROR] Could not write to the config file!");
        }
    }

}
