import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.lang.reflect.Method;
import java.util.Properties;

/**
 * The main class of Simple IDE. All of the fun stuff is done here.
 *
 * @author moparisthebest
 */
public class SIDE implements ActionListener {

    public final static String version = "0.7";

    public static SIDE _instance;

    private ProjectConfig prjConf = new ProjectConfig();

    private Properties SIDEConf;

    private JDKClass jdk = new JDKClass();

    private String frameTitle = "SIDE: Simple IDE";
    private String frameTitleAppend;

    private JFrame frame;
    private JScrollPane scroll;

    private boolean isHidden = false;

    /**
     * Entry point for the JVM.
     *
     * @param args Arguments to pass to the application.
     */
    public static void main(String[] args) {
        new SIDE(args);
    }

    /**
     * Constructor for SIDE.
     *
     * @param args Arguments to pass to the application.
     */
    public SIDE(String[] args) {
        _instance = this;
        SIDEConf = new Properties();
        try {
            SIDEConf.load(new FileInputStream(new File("SIDEconf")));
            String absPath = SIDEConf.getProperty("lastPrj");
            if (prjConf.loadSettings(absPath)) {
                frameTitleAppend = frameTitle + " -- Project: " + prjConf.get("name");
                jdk.setHomeDir(prjConf.get("homeDir"));
            }
        } catch (Exception e) {
        }
        initUI();
        log(frameTitle);
    }

    /**
     * This stores the last opened project in the applications
     * configuration file, so it can be automatically loaded
     * next startup.
     *
     * @param absPath
     */
    private void setLastPrj(String absPath) {
        SIDEConf.setProperty("lastPrj", absPath);
        try {
            SIDEConf.store(new FileOutputStream("SIDEconf"), "SIDE Config.");
        } catch (Exception e) {
        }
    }

    /**
     * Starts the UI, e.g the frame
     */
    private void initUI() {
        try {
            JFrame.setDefaultLookAndFeelDecorated(true);
            JPopupMenu.setDefaultLightWeightPopupEnabled(false);

            if (frameTitleAppend == null)
                frame = new JFrame(frameTitle);
            else
                frame = new JFrame(frameTitleAppend);
            frame.setLayout(new BorderLayout());

            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

            String[] menuTitles = {"File", "Console", "Configuration", "Help"};
            String[][] menuContent = {{"New Project","Quick Project", "Open Project", "-", "Garbage Collect", "-", "Quit"},
                    {"Hide", "Show"},
                    {"Edit MainClass", "Edit Parameters", "Edit ClassPath", "Edit Jar Settings"},
                    {"Info", "Website", "Forums", "Version"}};

            JMenuBar menuBar = makeMenuBar(menuTitles, menuContent);

            String[] panelButtons = new String[]{
                    "Compile", "Run", "JavaDoc", "Jar"
            };
            JPanel buttonPanel = makeButtonPanel(panelButtons);
            /*         String[] checkBoxes = new String[]{
                               "HP Above Heads", "Zoom/Camera Movement"
                       };

                       for (String name : checkBoxes) {
                           Checkbox menuItem = new Checkbox(name);
                           menuItem.addItemListener(this);
                           menuBar.add(menuItem);
                       }
            */
            JTextArea jTxtConsole = new JTextArea(20, 30);
            scroll = new JScrollPane(jTxtConsole, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            scroll.setPreferredSize(new Dimension(500, 150));
            jTxtConsole.setForeground(Color.white);
            jTxtConsole.setBackground(Color.black);
            jTxtConsole.setEditable(false);

            Console console = new Console(jTxtConsole, scroll);
            System.setOut(console);
            System.setErr(console);

            frame.getContentPane().add(menuBar, BorderLayout.NORTH);
            frame.getContentPane().add(buttonPanel, BorderLayout.CENTER);
            frame.getContentPane().add(scroll, BorderLayout.SOUTH);
            frame.pack();
            frame.setVisible(true);
        } catch (Exception e) {
            log("A general exception occured in initUI()");
            e.printStackTrace();
        }
    }

    /**
     * Creates a new JMenuBar from the given input. All menu contents are
     * added to this actionlistener.
     *
     * @param menuTitles  A String[] of the menu titles.
     * @param menuContent A String[][] of the menu contents, "-" adds a seperator
     * @return new JMenuBar already set up.
     */
    private JMenuBar makeMenuBar(String[] menuTitles, String[][] menuContent) {
        JMenuBar menuBar = new JMenuBar();
        for (int x = 0; x < menuTitles.length; ++x) {
            JMenu fileMenu = new JMenu(menuTitles[x]);
            for (String name : menuContent[x]) {
                JMenuItem menuItem = new JMenuItem(name);
                if (name.equalsIgnoreCase("-"))
                    fileMenu.addSeparator();
                else {
                    menuItem.addActionListener(this);
                    fileMenu.add(menuItem);
                }
            }
            menuBar.add(fileMenu);
        }
        return menuBar;
    }

    /**
     * Creates a new JPanel complete with JButtons. All JButtons are
     * added to this actionlistener.
     *
     * @param buttonNames The names of the JButtons you want in the panel.
     * @return new JPanel already set up.
     */
    private JPanel makeButtonPanel(String[] buttonNames) {
        JPanel buttonPanel = new JPanel(new FlowLayout());
        for (String name : buttonNames) {
            JButton button = new JButton(name);
            button.addActionListener(this);
            buttonPanel.add(button);
        }
        return buttonPanel;
    }

    /**
     * This simply prints a message to the console.
     *
     * @param message String to System.out.println
     */
    public static synchronized void log(String message) {
        System.out.println(message);
    }

    /**
     * Action performed, all actions are sent as a string into the doAction method.
     *
     * @see SIDE#doAction(String)
     */
    public void actionPerformed(ActionEvent evt) {
        doAction(evt.getActionCommand());
    }

    /**
     * This processes all commands and does what is requested.
     *
     * @param cmd The command to do in string format, ignores case.
     */
    public void doAction(String cmd) {
        if (cmd != null) {
            if (cmd.equalsIgnoreCase("Hide")) {
                if (isHidden)
                    log("The console is already hidden");
                else {
                    frame.remove(scroll);
                    frame.pack();
                    isHidden = true;
                }
            } else if (cmd.equalsIgnoreCase("Show")) {
                if (!isHidden)
                    log("The console is already Visible");
                else {
                    frame.add(scroll, BorderLayout.SOUTH);
                    frame.pack();
                    isHidden = false;
                }
            } else if (cmd.equalsIgnoreCase("Quit")) {
                log("Exiting...");
                System.exit(-1);
            } else if (cmd.equalsIgnoreCase("New Project")) {
                if (!newProject(false))
                    log("Creating new project failed, you probably dont have permissions to write to that folder.");
            } else if (cmd.equalsIgnoreCase("Quick Project")) {
                if (!newProject(true))
                    log("Creating new project failed, you probably dont have permissions to write to that folder.");
            } else if (cmd.equalsIgnoreCase("Open Project")) {
                openProject();
            } else if (cmd.equalsIgnoreCase("website")) {
                launchURL("http://www.moparisthebest.com");
            } else if (cmd.equalsIgnoreCase("forums")) {
                launchURL("http://www.moparisthebest.com/smf/");
            } else if (cmd.equalsIgnoreCase("JavaDoc")) {
                if (jdk.javaDoc(prjConf.get("srcDir")))
                    log("succesfully javadoc'd -- look to the command line for more info");
                else
                    log("javadoc'n failed -- look to the command line for more info");
            } else if (cmd.equalsIgnoreCase("Jar")) {
                jar();
            } else if (cmd.equalsIgnoreCase("Compile")) {
                try {
                    if (jdk.compile(prjConf.get("srcDir"), prjConf.get("binDir"), prjConf.get("classPath")))
                        log("succesfully compiled -- look to the command line for more info");
                    else
                        log("compilation failed -- look to the command line for more info");
                } catch (Exception e) {
                    log("didnt find class");
                }
            } else if (cmd.equalsIgnoreCase("Run")) {
                run();
            } else if (cmd.equalsIgnoreCase("Edit Jar Settings")) {
                String jar = JOptionPane.showInputDialog(frame, "Do you want the jar to include src, bin, or both: ", prjConf.get("jar"));
                if (jar != null)
                    if (jar.equals("src") || jar.equals("bin") || jar.equals("both"))
                        prjConf.set("jar", jar);
            } else if (cmd.equalsIgnoreCase("Garbage Collect")) {
                log("Collecting Garbage");
                System.gc();
            } else if (cmd.equalsIgnoreCase("Edit ClassPath")) {
                prjConf.set("classPath", JOptionPane.showInputDialog(frame, "Type in new classpath:\n( the delimiter is ':' )", prjConf.get("classPath")));
            } else if (cmd.equalsIgnoreCase("Edit MainClass")) {
                prjConf.set("mainClass", JOptionPane.showInputDialog(frame, "Type in new Main Class: ", prjConf.get("mainClass")));
            } else if (cmd.equalsIgnoreCase("Edit Parameters")) {
                prjConf.set("params", JOptionPane.showInputDialog(frame, "Type in new Main Class Parameters: ", prjConf.get("params")));
            } else if (cmd.equalsIgnoreCase("Version")) {
                JOptionPane.showMessageDialog(frame,
                        "This version: "+version
                        , "SIDE: Simple IDE Version", JOptionPane.PLAIN_MESSAGE);
            }else if (cmd.equalsIgnoreCase("Info")) {
                JOptionPane.showMessageDialog(frame,
                        "This open-source IDE can do anything you want it to, its open source.\n" +
                                "If you need help visit moparisthebest.com."
                        , "SIDE: Simple IDE Info", JOptionPane.PLAIN_MESSAGE);
            }
        }
    }

    /**
     * Dynamically loads all of the classes in the binary directory
     * and runs the specified main class via reflection.
     * TODO: find a better way to run the program so it doesnt run in the same jvm space, try to avoid system calls.
     */
    public void run() {
        try {
            ClassAggregator ca = new ClassAggregator(new File(prjConf.get("homeDir") + prjConf.get("binDir")), false);
            for (Class c : ca) {
                if (c.getName().equals(prjConf.get("mainClass"))) {
                    log("trying to run " + prjConf.get("mainClass"));
                    Class mainArgType[] = {(new String[0]).getClass()};
                    Method main = c.getMethod("main", mainArgType);
                    main.invoke("main", (Object) prjConf.get("params").split(" "));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log("uhoh, run messed up");
        }
    }

    /**
     * Packages all the files into a jar with a name of your choice.
     * This jar will be located in your home directory.
     * Follows the jar preference.
     */
    public void jar() {
        String name = JOptionPane.showInputDialog(frame, "Type in the name of the jar: ", prjConf.get("name"));
        boolean success;
        if (prjConf.get("jar").equals("both"))
            success = jdk.jar(name, prjConf.get("srcDir"), prjConf.get("mainClass"), prjConf.get("binDir"));
        else if (prjConf.get("jar").equals("src"))
            success = jdk.jar(name, prjConf.get("srcDir"), prjConf.get("mainClass"));
        else
            success = jdk.jar(name, prjConf.get("binDir"), prjConf.get("mainClass"));
        if (success)
            log("succesfully jar'd");
        else
            log("jar'n failed");
    }

    /**
     * Loads a project configuration of your choice.
     */
    public void openProject() {
        log("Opening Project");
        JFileChooser chooser = new JFileChooser();
        chooser.addChoosableFileFilter(new SIDEFilter());
        int returnVal = chooser.showOpenDialog(frame);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            String absPath = chooser.getSelectedFile().getAbsolutePath();
            System.out.println("You chose to open this file: " + absPath);
            if (prjConf.loadSettings(absPath)) {
                frame.setTitle(frameTitle + " -- Project: " + prjConf.get("name"));
                String newHome = absPath.replaceAll(chooser.getSelectedFile().getName(), "");
                prjConf.set("homeDir", newHome);
                jdk.setHomeDir(newHome);
                setLastPrj(absPath);
            } else
                log("[ERROR] Program configuration could not be obtained.  Make a new project.");
        }
    }

    /**
     * Creates a new project, asks all the necesary questions.
     * @param quick True to only ask required questions and default the rest, false to ask everything.
     * @return True if the new project was created succesfully, false otherwise.
     */
    public boolean newProject(boolean quick) {
        log("Starting New Project");
        String name = JOptionPane.showInputDialog(frame, "Type project name: ", "project1");
        String home = "";
        JFileChooser chooser = new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setDialogTitle("Choose the Project's home Directory");
        int returnVal = chooser.showOpenDialog(frame);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            home = chooser.getSelectedFile().getAbsolutePath();
            prjConf = new ProjectConfig(name, home);
            jdk.setHomeDir(home);
        } else
            return false;
        String src = "src";
        if(!quick)
        src = JOptionPane.showInputDialog(frame, "Type in source directory: ", src);
        if (chkDir(home + "/" + src))
            prjConf.set("srcDir", src);
        else
            return false;
        String bin = "bin";
        if(!quick)
        bin = JOptionPane.showInputDialog(frame, "Type in binary directory: ", bin);
        if (chkDir(home + "/" + bin))
            prjConf.set("binDir", bin);
        else
            return false;
        //prjConf.set("classPath", JOptionPane.showInputDialog(frame, "Type in the classpath: ", "bin"));
        //question abandoned, in preference to default value
        prjConf.set("classPath", prjConf.get("homeDir") + prjConf.get("binDir"));
        prjConf.set("mainClass", JOptionPane.showInputDialog(frame, "Type in the Main Class: "));
        String jar = "both";
        if(!quick)
        jar = JOptionPane.showInputDialog(frame, "Do you want the jar to include src, bin, or both: ", jar);
        if (jar != null)
            if (jar.equals("src") || jar.equals("bin") || jar.equals("both"))
                prjConf.set("jar", jar);
            else
                prjConf.set("jar", "both");
        setLastPrj(prjConf.get("homeDir") + prjConf.get("name") + ".side");
        frame.setTitle(frameTitle + " -- Project: " + prjConf.get("name"));
        return true;
    }

    /**
     * Checks to see if a file exists, and if not creates the directory.
     *
     * @param fileName Path to the directory.
     * @return True if file exists or was created, false otherwise.
     */
    private boolean chkDir(String fileName) {
        if (fileName != null)
            try {
                File f = new File(fileName);
                if (!f.exists())
                    f.mkdir();
                return true;
            }
            catch (Exception e) {
                log("Error making file: " + e);
                return false;
            }
        return false;
    }

    /**
     * Opens a URL in a new browser
     *
     * @param url The url to open
     */
    public static void launchURL(String url) {
        String osName = System.getProperty("os.name");
        try {
            if (osName.startsWith("Mac OS")) {
                Class fileMgr = Class.forName("com.apple.eio.FileManager");
                Method openURL = fileMgr.getDeclaredMethod("openURL", String.class);
                openURL.invoke(null, url);
            } else if (osName.startsWith("Windows"))
                Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
            else {
                String[] browsers = {
                        "firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape"};
                String browser = null;
                for (int count = 0; count < browsers.length && browser == null; count++)
                    if (Runtime.getRuntime().exec(new String[]{"which", browsers[count]}).waitFor() == 0)
                        browser = browsers[count];
                if (browser == null)
                    throw new Exception("Could not find web browser");
                else
                    Runtime.getRuntime().exec(new String[]{browser, url});
            }
        }
        catch (Exception e) {
            log("Error opening URL");
            e.printStackTrace();
        }
    }

}

class SIDEFilter extends FileFilter {

    public SIDEFilter() {
    }

    public boolean accept(File f) {
        String name = f.getName();
        return f.isDirectory() || (name.endsWith(".side") && name.indexOf("$") == -1);
    }

    public String getDescription() {
        return "SIDE Project Files";
    }
}